@@ -3,6 +3,7 @@ from os import PathLike
 from pathlib import Path
 from re import search
 from shutil import copy
+from shutil import copy2
 from sqlite3 import Connection
 from sqlite3 import Cursor as SQLCursor
 from sqlite3 import DatabaseError
@@ -92,8 +93,9 @@ def copy_cursors(db_dest: 'Database', cursors: Iterable['Cursor'], replace: bool
             raise DatabaseError(f"Unknown table {cursor.table.name}")
         for entry in cursor:
             if dest_table.name.lower() == db_dest.submissions.name.lower():
-                f, t = cursor_db.submissions.get_submission_files(entry[cursor.table.key.name])
-                db_dest.submissions.save_submission(entry, f.read_bytes() if f else None, t.read_bytes() if t else None,
+                fs, t = cursor_db.submissions.get_submission_files(entry[cursor.table.key.name])
+                db_dest.submissions.save_submission(entry, [f.read_bytes() for f in fs or []],
+                                                    t.read_bytes() if t else None,
                                                     replace=replace, exist_ok=exist_ok)
             else:
                 dest_table.insert(dest_table.format_entry(entry), replace=replace, exists_ok=True)
@@ -313,40 +315,61 @@ class SubmissionsTable(Table):
     def files_folder(self) -> Path:
         return self.database.settings.files_folder
 
-    def save_submission(self, submission: dict[str, Value | list[Value]], file: bytes = None, thumbnail: bytes = None,
-                        *, replace: bool = False, exist_ok: bool = False):
+    def save_submission(self, submission: dict[str, Value | list[Value]], files: list[bytes] = None,
+                        thumbnail: bytes = None, *, replace: bool = False, exist_ok: bool = False):
         submission = self.format_entry(submission)
+        file_url: list[str] = \
+            SubmissionsColumns.FILEURL.value.from_entry(submission[SubmissionsColumns.FILEURL.value.name])
 
-        submission[SubmissionsColumns.FILEEXT.value.name] = \
+        submission[SubmissionsColumns.FILEEXT.value.name] = SubmissionsColumns.FILEEXT.value.to_entry([
             self.save_submission_file(
                 submission[SubmissionsColumns.ID.value.name], file, "submission",
-                s[1] if (s := search(r"/[^/]+\.([^.]+)$", submission[SubmissionsColumns.FILEURL.value.name])) else "")
-        self.save_submission_thumbnail(submission[SubmissionsColumns.ID.name], thumbnail)
-        submission[SubmissionsColumns.FILESAVED.value.name] = (0b10 * bool(file)) + (0b01 * bool(thumbnail))
+                s[1] if (s := search(r"/[^/]+\.([^.]+)$", file_url[0] if file_url else "")) else "", n)
+            for n, file in enumerate(files) if file
+        ])
+        self.save_submission_thumbnail(submission[SubmissionsColumns.ID.name], thumbnail or None)
+
+        submission[SubmissionsColumns.FILESAVED.value.name] = (
+                (0b100 * all(map(bool, files)) * bool(files)) +  # all files were valid
+                (0b010 * any(map(bool, files))) +  # at least one file was valid
+                (0b001 * bool(thumbnail))  # the thumbnail was valid
+        )
 
         self.insert(submission, replace=replace, exists_ok=exist_ok)
 
-    def save_submission_file(self, submission_id: int, file: bytes | None, name: str, ext: str,
+    def save_submission_file(self, submission_id: int, file: bytes | None, name: str, ext: str, n: int = 0,
                              guess_ext: bool = True) -> str:
         if file is None:
             return ""
+        assert n >= 0, "n must be zero or positive"
 
         ext: str = guess_extension(file, ext) if guess_ext else ext
         folder: Path = self.files_folder / tiered_path(submission_id)
         folder.mkdir(parents=True, exist_ok=True)
-        folder.joinpath(name + f".{ext}" * bool(ext)).write_bytes(file)
+        folder.joinpath(f"{name}{n if n > 0 else ''}" + f".{ext}" * bool(ext)).write_bytes(file)
 
         return ext
 
     def save_submission_thumbnail(self, submission_id: int, file: bytes | None):
         self.save_submission_file(submission_id, file, "thumbnail", "jpg", False)
 
-    def get_submission_files(self, submission_id: int) -> tuple[Path | None, Path | None]:
+    def get_submission_files(self, submission_id: int) -> tuple[list[Path] | None, Path | None]:
         if (entry := self[submission_id]) is None or (f := entry[SubmissionsColumns.FILESAVED.value.name]) == 0:
             return None, None
         folder: Path = self.files_folder / tiered_path(submission_id)
-        file_ext: str = f".{(ext := entry[SubmissionsColumns.FILEEXT.value.name])}" * bool(ext)
-        return folder / f"submission{file_ext}" if f & 0b10 else None, folder / "thumbnail.jpg" if f & 0b01 else None
+        file_ext: list[str] = entry[SubmissionsColumns.FILEEXT.value.name]
+        return (
+            [folder / f"submission{n or ''}{('.' + ext) if ext else ''}"
+             for n, ext in enumerate(file_ext)] if f & 0b10 else None,
+            folder / "thumbnail.jpg" if f & 0b01 else None
+        )
+
+    def set_filesaved(self, submission_id: int, all_files: bool | int, any_file: bool | int, thumbnail: bool | int):
+        filesaved: int = (0b100 * bool(all_files)) + (0b010 * bool(any_file)) + (0b001 * bool(thumbnail))
+        if self._get_exists(submission_id)[SubmissionsColumns.FILESAVED.value.name] != filesaved:
+            self.update({EQ: {self.key.name: submission_id}}, {SubmissionsColumns.FILESAVED.value.name: filesaved})
+            return True
+        return False
 
     def set_folder(self, submission_id: int, folder: str) -> bool:
         if self._get_exists(submission_id)[SubmissionsColumns.FOLDER.value.name] != (folder := folder.lower().strip()):
@@ -416,7 +439,9 @@ class CommentsTable(Table):
 class SettingsTable(Table):
     _version_setting: str = "VERSION"
     _files_folder_setting: str = "FILESFOLDER"
+    _backup_folder_setting: str = "BACKUPFOLDER"
     _default_files_folder: str = "FA.files"
+    _default_backup_folder: str = "FA.backup"
 
     def __getitem__(self, item: str) -> str | None:
         return (super().__getitem__(item) or {}).get(SettingsColumns.SVALUE.value.name, None)
@@ -437,6 +462,20 @@ class SettingsTable(Table):
     def files_folder(self, value: str | Path):
         self[self._files_folder_setting] = str(value)
 
+    @property
+    def backup_folder(self) -> Path | None:
+        folder: str | None = self[self._backup_folder_setting]
+        if folder is None:
+            return None
+        return p if (p := Path(folder)).is_absolute() else (self.database.path.parent / p).resolve()
+
+    @backup_folder.setter
+    def backup_folder(self, value: str | Path | None):
+        if value is None:
+            del self[self._backup_folder_setting]
+        else:
+            self[self._backup_folder_setting] = str(value)
+
     def create(self, exists_ignore: bool = False):
         super().create(exists_ignore=exists_ignore)
         self.insert({SettingsColumns.SETTING.value.name: self._files_folder_setting,
@@ -587,5 +626,21 @@ class Database:
         copy_cursors(db_b, cursors or [self.users.select(), self.submissions.select(), self.journals.select()],
                      replace=replace, exist_ok=exist_ok)
 
+    def backup(self, *, folder: Path = None, date_format: str = "%Y-%m-%d %H.%M.%S"):
+        folder: Path | None = folder or self.settings.backup_folder
+        if folder is None:
+            raise ValueError("No backup folder set in database settings")
+        m_time: datetime = datetime.fromtimestamp(self.path.stat().st_mtime)
+        folder.mkdir(parents=True, exist_ok=True)
+        backup_file: Path = folder / (f"{self.path.name.removesuffix(self.path.suffix)} "
+                                      f"{m_time.strftime(date_format or '%Y-%m-%d %H.%M.%S')}"
+                                      f"{self.path.suffix}")
+        try:
+            copy2(self.path, backup_file.with_suffix(".tmp"))
+            copy2(backup_file.with_suffix(".tmp"), backup_file)
+        except Exception:
+            backup_file.with_suffix(".tmp").unlink(missing_ok=True)
+            raise
+
     def close(self):
         self.connection.close()
