@@ -315,6 +315,75 @@ def make_database_5_2_2(conn: Connection) -> Connection:
     return conn
 
 
+# noinspection SqlResolve,SqlNoDataSourceInspection,DuplicatedCode
+def make_database_5_3(conn: Connection) -> Connection:
+    conn.execute("""create table USERS
+    (USERNAME text unique not null check (length(USERNAME) > 0),
+    FOLDERS text not null,
+    ACTIVE boolean not null,
+    USERPAGE text not null,
+    primary key (USERNAME));""")
+
+    conn.execute("""create table SUBMISSIONS
+    (ID integer unique not null check (ID > 0),
+    AUTHOR text not null check (length(AUTHOR) > 0),
+    TITLE text not null,
+    DATE datetime not null,
+    DESCRIPTION text not null,
+    TAGS text not null,
+    CATEGORY text not null,
+    SPECIES text not null,
+    GENDER text not null,
+    RATING text not null,
+    TYPE text not null check (TYPE in ('image', 'music', 'text', 'flash')),
+    FILEURL text not null,
+    FILEEXT text not null,
+    FILESAVED integer not null check (FILESAVED in (0, 1, 2, 3, 4, 5, 6, 7)),
+    FAVORITE text not null,
+    MENTIONS text not null,
+    FOLDER text not null check (FOLDER in ('gallery', 'scraps')),
+    USERUPDATE boolean not null,
+    primary key (ID));""")
+
+    conn.execute("""create table JOURNALS
+    (ID integer unique not null check (ID > 0),
+    AUTHOR text not null check (length(AUTHOR) > 0),
+    TITLE text not null,
+    DATE datetime not null,
+    CONTENT text not null,
+    MENTIONS text not null,
+    USERUPDATE integer not null check (USERUPDATE in (0, 1)),
+    primary key (ID))""")
+
+    conn.execute("""create table COMMENTS
+    (ID integer not null check (ID > 0),
+    PARENT_TABLE text not null check (PARENT_TABLE in ('SUBMISSIONS', 'JOURNALS')),
+    PARENT_ID integer not null check (PARENT_ID > 0),
+    REPLY_TO integer check (REPLY_TO == null or REPLY_TO > 0),
+    AUTHOR text not null check (length(AUTHOR) > 0),
+    DATE datetime not null,
+    TEXT text not null,
+    primary key (ID, PARENT_TABLE, PARENT_ID))""")
+
+    conn.execute("""create table SETTINGS
+    (SETTING text unique not null check (length(SETTING) > 0),
+    SVALUE text check (SVALUE == null or length(SVALUE) > 0),
+    primary key (SETTING));""")
+
+    conn.execute("insert or ignore into SETTINGS (SETTING, SVALUE) values (?, ?)", ["FILESFOLDER", "FA.files"])
+    conn.execute("insert or ignore into SETTINGS (SETTING, SVALUE) values (?, ?)", ["BACKUPFOLDER", "FA.backup"])
+    conn.execute("insert or ignore into SETTINGS (SETTING, SVALUE) values (?, ?)", ["VERSION", "5.3.0"])
+
+    conn.execute("""create table HISTORY
+    (TIME datetime unique not null,
+    EVENT text not null,
+    primary key (TIME));""")
+
+    conn.commit()
+
+    return conn
+
+
 # noinspection SqlResolve,SqlNoDataSourceInspection,DuplicatedCode
 def update_5_0(conn: Connection, _db_path: Path, db_new_path: Path):
     make_database_5(connect(db_new_path)).close()
@@ -421,6 +490,58 @@ def update_5_2_2(conn: Connection, _db_path: Path, db_new_path: Path) -> list[st
     return []
 
 
+# noinspection SqlResolve,SqlNoDataSourceInspection,DuplicatedCode,SqlWithoutWhere
+def update_5_3(conn: Connection, _db_path: Path, db_new_path: Path) -> list[str]:
+    make_database_5_3(connect(db_new_path)).close()
+    conn.execute("attach database ? as db_new", [str(db_new_path)])
+    conn.execute("insert into db_new.USERS select * from USERS")
+    conn.execute("insert into db_new.SUBMISSIONS select * from SUBMISSIONS")
+    conn.execute("insert into db_new.JOURNALS select * from JOURNALS")
+    conn.execute("insert into db_new.COMMENTS select * from COMMENTS")
+    conn.execute("insert into db_new.HISTORY select * from HISTORY")
+    conn.execute("insert or replace into db_new.SETTINGS select * from SETTINGS where SETTING != 'VERSION'")
+    conn.execute("update db_new.SETTINGS set SVALUE = '5.3.0' where SETTING = 'VERSION'")
+    conn.execute("update db_new.SUBMISSIONS set FILEURL = ('|' || FILEURL || '|')")
+    conn.execute("update db_new.SUBMISSIONS set FILEEXT = ('|' || FILEEXT || '|') where FILESAVED & 2")
+    conn.execute("update db_new.SUBMISSIONS set FILESAVED = FILESAVED + 4 where FILESAVED & 2")
+    return []
+
+
+# noinspection SqlResolve,SqlNoDataSourceInspection,DuplicatedCode,SqlWithoutWhere
+def update_5_3_2(conn: Connection, _db_path: Path, db_new_path: Path) -> list[str]:
+    make_database_5_3(connect(db_new_path)).close()
+    conn.execute("attach database ? as db_new", [str(db_new_path)])
+    conn.execute("insert into db_new.USERS select * from USERS")
+    conn.execute("insert into db_new.SUBMISSIONS select * from SUBMISSIONS")
+    conn.execute("insert into db_new.JOURNALS select * from JOURNALS")
+    conn.execute("insert into db_new.COMMENTS select * from COMMENTS")
+    conn.execute("insert into db_new.HISTORY select * from HISTORY")
+    conn.execute("insert or replace into db_new.SETTINGS select * from SETTINGS where SETTING != 'VERSION'")
+    conn.execute("update db_new.SETTINGS set SVALUE = '5.3.2' where SETTING = 'VERSION'")
+
+    def tiered_path(i: int | str, depth: int = 5, width: int = 2) -> Path:
+        id_str: str = str(int(i)).zfill(depth * width)
+        return Path(*[id_str[n_:n_ + width] for n_ in range(0, depth * width, width)])
+
+    files_folder: Path = Path(conn.execute("select SVALUE from SETTINGS where SETTING = 'FILESFOLDER'").fetchone()[0])
+    submissions = conn.execute("""select ID, FILEEXT from db_new.SUBMISSIONS
+        where FILEEXT like '%|||%' or FILEEXT like '%||' order by ID""")
+    submissions_fixed: int = 0
+
+    for [id_, exts_raw] in submissions:
+        exts = exts_raw.removeprefix("|").removesuffix("|").split("||")
+        folder: Path = files_folder / tiered_path(id_)
+        for n, ext in [(n, e) for n, e in enumerate(exts) if "|" in e]:
+            ext_new = ext.removesuffix("|")
+            file = folder / f"submission{n if n else ''}.{ext}"
+            if file.is_file():
+                file.replace(file.with_suffix(ext_new))
+            exts[n] = ext_new
+        conn.execute("update db_new.SUBMISSIONS set FILEEXT = ? where ID = ?", [f"|{'|'.join(exts)}|", id_])
+        submissions_fixed += 1
+    return [f"{submissions_fixed} submissions extensions fixed"]
+
+
 def update_wrapper(conn: Connection, update_function: Callable[[Connection, Path, Path], list[str] | None],
                    version_old: str, version_new: str) -> Connection:
     print(f"Updating {version_old} to {version_new}... ", end="", flush=True)
@@ -482,6 +603,10 @@ def update_database(conn: Connection, version: str) -> Connection:
         conn = update_wrapper(conn, update_5_2, db_version, v)  # 5.1.2 to 5.2.0
     elif compare_versions(db_version, v := "5.2.2") < 0:
         conn = update_wrapper(conn, update_5_2_2, db_version, v)  # 5.2.0-5.2.1 to 5.2.2
+    elif compare_versions(db_version, v := "5.3.0") < 0:
+        conn = update_wrapper(conn, update_5_3, db_version, v)  # 5.2.2 to 5.3.0
+    elif compare_versions(db_version, v := "5.3.2") < 0:
+        conn = update_wrapper(conn, update_5_3_2, db_version, v)  # 5.3.0 to 5.3.2
     elif compare_versions(db_version, version) < 0:
         return update_patch(conn, db_version, version)  # Update to the latest patch
 
